﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraEvent : MonoBehaviour {
	/// <summary>
	/// Moves the camera to an end location. All information is passed in through the cutsceneEvent struct
	/// </summary>
	/// <returns>Nothing</returns>
	/// <param name="cutsceneEvent">the CutsceneEvent you want to use</param>
	public IEnumerator cameraAction(CutsceneEvent cutsceneEvent) {
		//assigns the camera we want to use and the rigidbody attached to that camera.
		//Currently is only assigned to the main camera. More functionality is needed
		//to create a cutscene camera.
		Camera cutsceneCamera = Camera.main;
		Rigidbody cameraRigidbody = cutsceneCamera.GetComponent<Rigidbody> ();
		//sets the velocity of the camera to a normalized direction vector, with a magnitude of the speed passed in through the CutsceneEvent
		cameraRigidbody.velocity = (cutsceneEvent.endLocation - new Vector3(cutsceneCamera.transform.position.x, cutsceneCamera.transform.position.y,  cutsceneCamera.transform.position.z)).normalized * cutsceneEvent.speed;
		//setting the size to -1 means the camera will stay zoomed at its current position
        if (cutsceneEvent.size == -1) {
			cutsceneEvent.size = cutsceneCamera.orthographicSize;
		}
        //setting the speed to -1 means the movement will happen instantaneously
        if(cutsceneEvent.speed == -1)
        {
            cutsceneCamera.transform.position = cutsceneEvent.endLocation;
        }
        //setting the zoom speed to -1 means the zoom will happen instantaneously
        if(cutsceneEvent.zoomSpeed == -1)
        {
            cutsceneCamera.orthographicSize = cutsceneEvent.size;
        }
        //checks if the cutscene has to wait for this action to complete
        if (cutsceneEvent.wait) {
			//if yes, start a new coroutine and make this coroutine wait for that coroutine to complete
			yield return StartCoroutine (completeCameraAction (cutsceneCamera, cutsceneEvent.endLocation, cutsceneEvent.size, cutsceneEvent.zoomSpeed, cameraRigidbody));
		} else {
			//if no, start a new coroutine and let this coroutine continue on as normal
			StartCoroutine (completeCameraAction (cutsceneCamera, cutsceneEvent.endLocation, cutsceneEvent.size, cutsceneEvent.zoomSpeed, cameraRigidbody));
		}
		//end this coroutine
		yield return 0;
	}

	public IEnumerator completeCameraAction(Camera camera, Vector3 endPosition, float cameraSize, float cameraZoomSpeed, Rigidbody cameraRigidbody) {
		//2 completed booleans to check both if the camera has been moved correctly and if the camera has zoomed properly
		bool moveCompleted = false;
		bool zoomCompleted = false;
		//while the character has still not reached the end of its movement or zooming
		while (!moveCompleted || !zoomCompleted) {
			//check if the camera has reached the end of its movement
			if (camera.transform.position.x >= endPosition.x - 0.5 && camera.transform.position.x <= endPosition.x + 0.5) {
				if (camera.transform.position.y >= endPosition.y - 0.5 && camera.transform.position.y <= endPosition.y + 0.5) {
                    if(camera.transform.position.z >= endPosition.z - 0.5 && camera.transform.position.z <= endPosition.z + 0.5)
                    {
                        //signal to the while loop that it has completed its process
                        moveCompleted = true;
                        //stop the character from moving
                        cameraRigidbody.velocity = Vector2.zero;
                    }
				}
			}
			//check if the camera has been zoomed to the correct position
			if (camera.orthographicSize < cameraSize + 0.5 && camera.orthographicSize > cameraSize - 0.5) {
				zoomCompleted = true;
			//does the camera have to zoom in?
			} else if (camera.orthographicSize > cameraSize) {
				//zoom the camera in by a magnitude of the zoom value
				camera.orthographicSize = camera.orthographicSize - cameraZoomSpeed;
			//does the camera have to zoom out?
			} else if (camera.orthographicSize < cameraSize) {
				//zoom the camera out by a magnitude of the zoom value
				camera.orthographicSize = camera.orthographicSize + cameraZoomSpeed;
			}
			yield return new WaitForEndOfFrame ();
		}
		yield return 0;
	}
}
